function [Y, I, J] = shake(X, DIM)
% SHAKE - Randomize a matrix along a specific dimension
%   Y = SHAKE(X) randomizes the order of the elements in each column of the
%   2D matrix. For N-D matrices it randomizes along the first non-singleton
%   dimension.
%
%   SHAKE(X, DIM) randomizes along the dimension DIM.
%
%   [Y, I, J] = SHAKE(X) returns indices so that Y = X(I) and X = Y(J).
%
%   Example:
%     A = [1 2 3 ; 4 5 6 ; 7 8 9 ; 10 11 12] ; % see <SLM> on the FEX ...
%     B = shake(A, 2)  % -> each row is shaked randomly
%      %  3     2     1
%      %  6     4     5
%      %  7     8     9
%      % 11    10    12%   
%     C = sort(B, 2) % -> equals A!
%
%     For vectors V, shake(V) is similar to V(randperm(numel(V)).
%
%     The function of SHAKE can be thought of as holding a matrix and shake
%     in a particular direction (dimension), so that elements are getting
%     shuffled within that direction only.
%
%   See also RAND, SORT, RANDPERM
%   and RANDSWAP on the File Exchange

% for Matlab R2018a
% version 5.0 (feb 2019)
% (c) Jos van der Geest
% email: samelinoa@gmail.com

% History
% Created: dec 2005
% Revisions
% 1.1 : changed the meaning of the DIM. Now DIM==1 works along the rows, preserving
% columns, like in <sort>.
% 2.0 (aug 2006) : randomize along any dimension
% 2.1 (aug 2006) : output indices argument
% 3.0 (oct 2006) : new & easier algorithm
% 4.0 (dec 2006) : fixed major error in 3.0
% 4.1 (may 2008) : fixed error for scalar input
% 5.0 (feb 2019) : updated for newer releases of ML; now uses permute

narginchk(1,2) ;
szX = size(X) ;

if nargin==1 
    DIM = find(szX>1, 1) ;
elseif (numel(DIM) ~= 1) || (fix(DIM) ~= DIM) || (DIM < 1)
    error('Shake:DimensionError','Dimension argument must be a positive integer scalar.') ;
end

% we will shake the indices
I = reshape(1:numel(X),szX) ;

if numel(X) < 2 || DIM > ndims(X) || szX(DIM) < 2    
    % in some cases, do nothing
else
    % put the dimension of interest first
    pdv = [DIM, 1:DIM-1, DIM+1:ndims(I)] ;
    I = permute(I, pdv) ;
    % reshape it into a 2D matrix, we'll randomize along rows
    I = reshape(I, szX(DIM), []) ;
    [~, ri] = sort(rand(size(I)), 1) ;      % get new random row indices
    ci = repmat(1:size(I,2), szX(DIM), 1) ; % but keep old column indices
    I = I(sub2ind(size(I), ri, ci)) ;       % get new indices    
    I = reshape(I, szX(pdv)) ; % reshape into 
    I = ipermute(I, pdv) ; % restore dimensions
end

% re-index
Y = X(I) ; 

if nargout==3
    J = zeros(size(X)) ;
    J(I) = 1:numel(J) ;
end

% example plot
% A = repmat(1:3,10,2) ;
% subplot(1,2,1) ; imagesc(A) ;
% title('A')
% subplot(1,2,2) ; imagesc(shake(A,2)) ;
% title('shake(A,2)') ;

